@@ -1,5 +1,6 @@
 import base64
 import binascii
+import string
 from decimal import Decimal
 from . import strutils
 from . import randomutils
@@ -12,6 +13,9 @@ from .aesutils import padding_iso10126 as aes_padding_iso10126
 from .aesutils import padding_pkcs5 as aes_padding_pkcs5
 
 
+class DecryptFailed(RuntimeError):
+    pass
+
 class S12CipherCore(object):
 
     def __init__(self, password):
@@ -70,6 +74,75 @@ def s12_decrypt(data, password):
     return cipher.decrypt(data)
 
 
+class MappingCipherBase(object):
+    def __init__(self, password):
+        self.password = password
+        self.randomGenerator = randomutils.Random(self.password)
+        self.seeds = self.get_seeds()
+        self.encrypt_mapping = self.get_encrypt_mapping()
+        self.decrypt_mapping = self.get_decrypt_mapping()
+
+    def get_seeds(self):
+        raise NotImplementedError()
+
+    def get_encrypt_mapping(self):
+        mapping = {}
+        for i in range(256):
+            mapping[bytes([i])] = self.seeds[i]
+        return mapping
+    
+    def get_decrypt_mapping(self):
+        mapping = {}
+        for i in range(256):
+            mapping[self.seeds[i]] = bytes([i])
+        return mapping
+    
+    def encrypt(self, data, **kwargs):
+        if data is None:
+            return None
+        result = b""
+        data = strutils.force_bytes(data)
+        for c in data:
+            result += self.encrypt_mapping[bytes([c])]
+        return result
+    
+    def decrypt(self, data, **kwargs):
+        if data is None:
+            return None
+        result = b""
+        data_length = len(data)
+        max_seed_length = max([len(x) for x in self.decrypt_mapping.keys()])
+        start = 0
+        while start < data_length:
+            found = False
+            for seed_length in range(1, max_seed_length+1):
+                seed = data[start: start+seed_length]
+                if seed in self.decrypt_mapping:
+                    result += self.decrypt_mapping[seed]
+                    start += seed_length
+                    found = True
+                    break
+            if not found:
+                raise DecryptFailed()
+        return result
+
+class S1CipherCore(MappingCipherBase):
+    def get_seeds(self):
+        seeds = list(range(256))
+        self.randomGenerator.shuffle(seeds)
+        return [bytes([x]) for x in seeds]
+
+class S2CipherCore(MappingCipherBase):
+    def get_seeds(self):
+        letters = string.ascii_lowercase
+        seeds = set()
+        for a in letters:
+            for b in letters:
+                seeds.add(a+b)
+        seeds = list(seeds)
+        self.randomGenerator.shuffle(seeds)
+        seeds = [x.encode() for x in seeds[:256]]
+        return seeds
 
 class IvCipherCore(object):
     def __init__(self, password):
@@ -116,7 +189,7 @@ class IvCipherCore(object):
                     r = a * n + delta + 1
                     break
             if t != number:
-                raise RuntimeError("iv_decrypt failed: number={}".format(number))
+                raise DecryptFailed("iv_decrypt failed: number={}".format(number))
         if flag:
             r = -1 * r
         return r
@@ -162,7 +235,7 @@ class IvfCipherCore(IvCipherCore):
         else:
             return number
 
-class EncoderBase(object):
+class ResultEncoderBase(object):
 
     def encode(self, data):
         raise NotImplementedError()
@@ -170,7 +243,7 @@ class EncoderBase(object):
     def decode(self, data):
         raise NotImplementedError()
 
-class RawDataEncoder(EncoderBase):
+class RawDataEncoder(ResultEncoderBase):
 
     def encode(self, data):
         return data
@@ -178,7 +251,7 @@ class RawDataEncoder(EncoderBase):
     def decode(self, data):
         return data
 
-class HexlifyEncoder(EncoderBase):
+class HexlifyEncoder(ResultEncoderBase):
 
     def encode(self, data):
         if data is None:
@@ -192,7 +265,7 @@ class HexlifyEncoder(EncoderBase):
         data = strutils.force_bytes(data)
         return binascii.unhexlify(data)
 
-class Base64Encoder(EncoderBase):
+class Base64Encoder(ResultEncoderBase):
 
     def encode(self, data):
         if data is None:
@@ -206,7 +279,7 @@ class Base64Encoder(EncoderBase):
         data = strutils.force_bytes(data)
         return base64.decodebytes(data)
 
-class SafeBase64Encoder(EncoderBase):
+class SafeBase64Encoder(ResultEncoderBase):
 
     def encode(self, data):
         if data is None:
@@ -220,6 +293,14 @@ class SafeBase64Encoder(EncoderBase):
         data = strutils.force_bytes(data)
         return base64.urlsafe_b64decode(data)
 
+class Utf8Encoder(ResultEncoderBase):
+    
+    def encode(self, data):
+        return data.decode("utf-8")
+
+    def decode(self, data):
+        return data.encode("utf-8")
+
 class CipherBase(object):
 
     def get_defaults(self):
@@ -238,7 +319,7 @@ class CipherBase(object):
         self.kwargs = params.get("kwargs", {})
         if self.password:
             self.kwargs.update({"password": self.password})
-        self.encoder = params["encoder"]
+        self.result_encoder = params["result_encoder"]
         self.force_text = params.get("force_text", False)
         self.text_encoding = params.get("text_encoding", "utf-8")
         self.cipher_core_class = params.get("cipher_core", None)
@@ -256,9 +337,8 @@ class CipherBase(object):
         params = {}
         params.update(self.kwargs)
         params.update(self.encrypt_kwargs)
-        data = strutils.force_bytes(data, self.text_encoding)
         encrypted_data = self._encrypt(data, **params)
-        return self.encoder.encode(encrypted_data)
+        return self.result_encoder.encode(encrypted_data)
 
     def decrypt(self, text):
         if text is None:
@@ -266,7 +346,7 @@ class CipherBase(object):
         params = {}
         params.update(self.kwargs)
         params.update(self.decrypt_kwargs)
-        data = self.encoder.decode(text)
+        data = self.result_encoder.decode(text)
         decrypted_data = self._decrypt(data, **params)
         if self.force_text:
             return strutils.force_text(decrypted_data, self.text_encoding)
@@ -276,7 +356,7 @@ class CipherBase(object):
 class AesCipher(CipherBase):
     
     defaults = {
-        "encoder": RawDataEncoder(),
+        "result_encoder": RawDataEncoder(),
         "encrypt": aesutils.encrypt,
         "decrypt": aesutils.decrypt,
     }
@@ -284,24 +364,38 @@ class AesCipher(CipherBase):
 class S12Cipher(CipherBase):
 
     defaults = {
-        "encoder": RawDataEncoder(),
+        "result_encoder": RawDataEncoder(),
         "cipher_core": S12CipherCore,
     }
 
 class IvCipher(CipherBase):
 
     defaults = {
-        "encoder": RawDataEncoder(),
+        "result_encoder": RawDataEncoder(),
         "cipher_core": IvCipherCore,
     }
 
 class IvfCipher(CipherBase):
 
     defaults = {
-        "encoder": RawDataEncoder(),
+        "result_encoder": RawDataEncoder(),
         "cipher_core": IvfCipherCore,
         "kwargs": {
             "int_digits": 12,
             "float_digits": 4,
         }
     }
+
+class S1Cipher(CipherBase):
+
+    defaults = {
+        "result_encoder": RawDataEncoder(),
+        "cipher_core": S1CipherCore,
+    }
+
+class S2Cipher(CipherBase):
+
+    defaults = {
+        "result_encoder": Utf8Encoder(),
+        "cipher_core": S2CipherCore,
+    }
