@@ -506,3 +506,28 @@ def format_with_mapping(template, mapping, *args, **kwargs):
             value = trans(kwargs[key])
         new_kwargs[key] = value
     return template.format(*new_args, **new_kwargs)
+
+
+default_quotes = [
+    '"""',
+    "'''",
+    '"',
+    "'",
+    "`",
+    ('“', '”'),
+    ("‘", "’"),
+    ("『", "』"),
+    ("「", "」"),
+    ("﹁", "﹂"),
+    ("﹃", "﹄"),
+    ("｢", "｣"),
+    ]
+
+def unquote(value, quotes=None):
+    quotes = quotes or default_quotes
+    for quote_pair in quotes:
+        if isinstance(quote_pair, str):
+            quote_pair = (quote_pair, quote_pair)
+        if value.startswith(quote_pair[0]) and value.endswith(quote_pair[1]):
+            return value[len(quote_pair[0]):-1*len(quote_pair[1])]
+    return value
